@@ -2,8 +2,9 @@ from statefun_tasks.messages_pb2 import TaskAction, TaskStatus
 
 
 class _FlinkAction(object):
-    def __init__(self, context):
+    def __init__(self, context, pipeline):
         self._context = context
+        self._pipeline = pipeline
         
     def run(self, action_request):
         if action_request.action == TaskAction.GET_STATUS:
@@ -15,6 +16,12 @@ class _FlinkAction(object):
         elif action_request.action == TaskAction.GET_RESULT:
             return self._get_task_result()
 
+        elif action_request.action == TaskAction.PAUSE_PIPELINE:
+            return self._pause_pipeline()
+
+        elif action_request.action == TaskAction.RESUME_PIPELINE:
+            return self._resume_pipeline()
+
         else:    
             raise ValueError(f'Unsupported task action {TaskAction.Name(action_request.action)}')
 
@@ -38,6 +45,7 @@ class _FlinkAction(object):
 
 
     def _get_task_result(self):
+
         task_result = self._context.storage.task_result or None
         if task_result is not None:
             return task_result
@@ -47,3 +55,17 @@ class _FlinkAction(object):
             return task_exception
 
         raise ValueError(f'Task result not found')
+
+    def _pause_pipeline(self):
+        if self._pipeline is None:
+            raise ValueError('Task is not a pipeline')
+        
+        if self._get_task_status().status != TaskStatus.Status.PENDING:
+            raise ValueError('Pipeline has already completed')
+            
+        return
+
+        raise ValueError(f'Not yet implemented')
+
+    def _resume_pipeline(self):
+        raise ValueError(f'Not yet implemented')
