@@ -1,6 +1,6 @@
 from statefun_tasks.context import TaskContext
 from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, Pipeline, TupleOfAny
-from statefun_tasks.pipeline_helper import _PipelineHelper
+from statefun_tasks.pipeline_behaviour import _PipelineBehaviour
 from statefun_tasks.serialisation import DefaultSerialiser, pack_any
 from statefun_tasks.types import Task, Group
 
@@ -12,7 +12,7 @@ class _Pipeline(object):
         self._pipeline = pipeline
         self._serialiser = serialiser if serialiser is not None else DefaultSerialiser()
         self._is_fruitful = is_fruitful
-        self._pipeline_helper = _PipelineHelper(self._pipeline, self._serialiser)
+        self._behaviour = _PipelineBehaviour(self._pipeline, self._serialiser)
 
     def to_proto(self) -> Pipeline:
         pipeline = Pipeline(entries=[p.to_proto(self._serialiser) for p in self._pipeline])
@@ -31,7 +31,7 @@ class _Pipeline(object):
         return _Pipeline(pipeline, serialiser)
 
     def is_empty(self):
-        return not any(self._pipeline_helper.get_initial_tasks())
+        return not any(self._behaviour.get_initial_tasks())
 
     def begin(self, context: TaskContext, invoking_task: TaskRequest, task_state):
         caller_id = context.get_caller_id()
@@ -41,6 +41,7 @@ class _Pipeline(object):
         context.pipeline_state.address = context.get_address()
         context.pipeline_state.pipeline.CopyFrom(self.to_proto())
         context.pipeline_state.is_fruitful = self._is_fruitful
+        context.pipeline_state.task_state.CopyFrom(pack_any(self._serialiser.to_proto(task_state)))
 
         if caller_id is not None:
             context.pipeline_state.caller_id = caller_id
@@ -51,44 +52,46 @@ class _Pipeline(object):
         context.pipeline_state.root_address = invoking_task.meta['root_pipeline_address'] or context.pipeline_state.address
 
         # 2. get initial tasks(s) to call - might be single start of chain task or a group of tasks to call in parallel
-        tasks = self._pipeline_helper.get_initial_tasks()
+        tasks, max_parallelism = self._behaviour.get_initial_tasks()
 
-        # 3. call each task
-        for task in tasks:
-            request = TaskRequest(id=task.task_id, type=task.task_type)
+        # 3. split into tasks to call now and those to defer if max parallelism is exceeded
+        tasks_to_call = self._behaviour.defer_tasks(context, tasks, max_parallelism=max_parallelism)
 
-            # set extra pipeline related parameters
-            self._add_pipeline_meta(context, caller_id, request)
+        # 4. call each task
+        for task, task_request in tasks_to_call:
+            context.send_message(task.get_destination(), task.task_id, task_request)
 
-            self._serialiser.serialise_request(request, task.request, state=task_state, retry_policy=task.retry_policy)
-
-            context.send_message(task.get_destination(), task.task_id, request)
 
     def resume(self, context: TaskContext, task_result_or_exception: Union[TaskResult, TaskException]):
         caller_id = context.get_caller_id()
-        #task_results = context.pipeline_state.task_results
 
-        # mark pipeline step as complete & record task result
-        self._pipeline_helper.mark_task_complete(caller_id, task_result_or_exception)
+        # mark pipeline step as complete
+        self._behaviour.mark_task_complete(caller_id, task_result_or_exception)
+
+        # release deferred tasks if they can now run
+        self._behaviour.release_deferred_tasks(context, caller_id, task_result_or_exception)
 
         # get the next step of the pipeline to run (if any)
-        _, next_step, group = self._pipeline_helper.get_next_step_in_pipeline(caller_id)
+        _, next_step, group = self._behaviour.get_next_step_in_pipeline(caller_id)
 
         # if this task is part group then we need to record the results so we can aggregate later
         if group is not None:
-            self._pipeline_helper.add_to_task_results(caller_id, task_result_or_exception, context.pipeline_state.task_results)
+            self._behaviour.add_to_task_results(caller_id, task_result_or_exception, context.pipeline_state.task_results)
 
             # once the group is complete aggregate the results
             if group.is_complete():
-                task_result_or_exception = self._pipeline_helper.aggregate_group_results(group, context.pipeline_state.task_results)
+                task_result_or_exception = self._behaviour.aggregate_group_results(group, context.pipeline_state.task_results)
+
 
         # if we got an exception then the next step is the finally_task if there is one (or none otherwise)
         if isinstance(task_result_or_exception, TaskException):
-            next_step = self._pipeline_helper.try_get_finally_task(caller_id)
+            next_step = self._behaviour.try_get_finally_task(caller_id)
 
         # turn next step into remainder of tasks to call
         if isinstance(next_step, Task):
-            remainder = [next_step]
+            tasks = [next_step]
+            max_parallelism = 1
+
             if next_step.is_finally:
                 # record the result of the task prior to the finally task so we can return it once the finally task completes
 
@@ -97,32 +100,20 @@ class _Pipeline(object):
                     else context.pipeline_state.result_before_finally
 
                 before_finally.CopyFrom(task_result_or_exception)
-
+            
         elif isinstance(next_step, Group):
-            remainder = self._pipeline_helper.get_initial_tasks(next_step)
+            tasks, max_parallelism = self._behaviour.get_initial_tasks(next_step)
         else:
-            remainder = []
+            tasks = []
 
-        # call next steps (if any)
-        if any(remainder):
-            task_result, task_state = self._serialiser.unpack_response(task_result_or_exception)
+        if any(tasks):
+            # split into tasks to call now and those to defer if max parallelism is exceeded
+            tasks_to_call = self._behaviour.defer_tasks(context, tasks, task_result_or_exception=task_result_or_exception, max_parallelism=max_parallelism)
 
-            for task in remainder:
-                task_id = task.task_id
-                task_args_and_kwargs = self._serialiser.to_args_and_kwargs(task.request)
-                
-                # extend task_result with any args & kwargs passed to the task explicitly
-                # noting that args from previous tasks are not passed to finally
-                task_request = self._serialiser.merge_args_and_kwargs(task_result if not task.is_finally else TupleOfAny(), task_args_and_kwargs)
+            # call the tasks
+            for task, task_request in tasks_to_call:
+                context.send_message(task.get_destination(), task.task_id, task_request)
 
-                request = TaskRequest(id=task_id, type=task.task_type)
-
-                # set extra pipeline related parameters
-                self._add_pipeline_meta(context, caller_id, request)
-
-                self._serialiser.serialise_request(request, task_request, state=task_state, retry_policy=task.retry_policy)
-
-                context.send_message(task.get_destination(), task_id, request)
         else:
             last_step = self._pipeline[-1]
 
@@ -156,7 +147,7 @@ class _Pipeline(object):
             'result' if isinstance(task_result_or_exception, TaskResult) else 'error')
 
         # pass back any state that we were given at the start of the pipeline
-        task_result_or_exception.state.CopyFrom(task_request.state)
+        task_result_or_exception.state.CopyFrom(context.pipeline_state.task_state)
 
         # finally emit the result (to egress, destination address or caller address)
         self._emit_result(context, task_request, task_result_or_exception)
@@ -187,14 +178,3 @@ class _Pipeline(object):
             if context.pipeline_state.caller_id != context.get_caller_id():  # don't call back to self
                 context.send_message(context.pipeline_state.caller_address, context.pipeline_state.caller_id,
                                      task_result_or_exception)
-
-    @staticmethod
-    def _add_pipeline_meta(context, caller_id, task_request: TaskRequest):
-        task_request.meta['pipeline_address'] = context.pipeline_state.address
-        task_request.meta['pipeline_id'] = context.pipeline_state.id
-        task_request.meta['root_pipeline_id'] = context.pipeline_state.root_id
-        task_request.meta['root_pipeline_address'] = context.pipeline_state.root_address
-
-        if caller_id is not None:
-            task_request.meta['parent_task_address'] = context.get_caller_address()
-            task_request.meta['parent_task_id'] = caller_id
